
C Program	P D B _ E S

C Long overdue coordinate automation PDB -> ES.
C Uses the atoms_dat routines to  read through the atoms_es.dat dictionary.

C Jim Warwicker, Manchester, April 2002.

C Get parameters from the environment variables.

C Modified March.03 to try and match between atyp fields to left in PDB
C relative to atoms_es.dat and res fields to right in PDB rel to atoms_es.

C Mod July 03 to have all TYR residues ionisable (stay TYR) or fixed/OH (TYRH)
C Under the control of TYR_IONISABLE env variable (default fixed OH).

C Mod Nov 03 to update MSE/HETATM to MET/ATOM.

	INTEGER		buf_model (100,2)
	CHARACTER*60	molecule_coords, waters, other_hetatm,
     1			protein, nucleic, tyr_ionisable,
     2			keep_model, same_chain, insert_renumber
	CHARACTER*80	file_in, file_out
	LOGICAL		protein_out, nucleic_out, other_out,
     1			tyr_ionise, keep_mdl, keep_chain, renumber

	COMMON	/files/		file_in, file_out
	COMMON	/params/	other_out, protein_out, nucleic_out,
     1				tyr_ionise, keep_chain, renumber
	COMMON	/mdl_numbers/	buf_model, keep_mdl

	CALL	GETENV ('MOLECULE_COORDS', molecule_coords)
	CALL	leader (molecule_coords, imc)
	file_in		= molecule_coords (1:imc)//'.pdb'
	file_out	= molecule_coords (1:imc)//'_es.pdb'

	CALL	GETENV ('WATERS', waters)
	CALL	leader (waters, iw)
	IF	(waters (1:1) .EQ. 'T') THEN
	  STOP 'WATERS not in dictionary yet'
	ELSE IF (waters (1:1) .NE. 'F') THEN
	  STOP 'WATERS not F'
	END IF

	CALL	GETENV ('OTHER_HETATM', other_hetatm)
	CALL	leader (other_hetatm, ioh)
	IF	(other_hetatm (1:1) .EQ. 'T') THEN
	  other_out	= .TRUE.
	ELSE IF (other_hetatm (1:1) .EQ. 'F') THEN
	  other_out	= .FALSE.
	ELSE
	  STOP 'OTHER not T/F'
	END IF


	CALL	GETENV ('PROTEIN', protein)
	CALL	leader (protein, ip)
	IF	(protein (1:1) .EQ. 'T') THEN
	  protein_out	= .TRUE.
	ELSE IF (protein (1:1) .EQ. 'F') THEN
	  protein_out	= .FALSE.
	ELSE
	  STOP 'PROTEIN not T/F'
	END IF

	CALL	GETENV ('NUCLEIC', nucleic)
	CALL	leader (nucleic, in)
	IF	(nucleic (1:1) .EQ. 'T') THEN
	  nucleic_out	= .TRUE.
	ELSE IF (nucleic (1:1) .EQ. 'F') THEN
	  nucleic_out	= .FALSE.
	ELSE
	  STOP 'NUCLEIC not T/F'
	END IF

	CALL	GETENV ('TYR_IONISABLE', tyr_ionisable)
	CALL	leader (tyr_ionisable, iti)
	IF	((tyr_ionisable (1:1) .EQ. 'T').OR.
     1		 (tyr_ionisable (1:3) .EQ. 'yes')) THEN
	  tyr_ionise	= .TRUE.
	ELSE
	  tyr_ionise	= .FALSE.
	END IF

	CALL	GETENV ('KEEP_MODEL', keep_model)
	CALL	leader (keep_model, ikm)
	IF	(keep_model (1:1) .EQ. 'T') THEN
	  keep_mdl	= .TRUE.
	ELSE
	  keep_mdl	= .FALSE.
	END IF

	CALL	GETENV ('SAME_CHAIN', same_chain)
	CALL	leader (same_chain, isc)
	IF	((same_chain (1:1) .EQ. 'T').OR.
     1		 (same_chain (1:3) .EQ. 'yes')) THEN
	  keep_chain	= .TRUE.
	ELSE
	  keep_chain	= .FALSE.
	END IF

	CALL	GETENV ('INSERT_RENUMBER', insert_renumber)
	CALL	leader (insert_renumber, iir)
	IF	((insert_renumber (1:1) .EQ. 'T').OR.
     1		 (insert_renumber (1:3) .EQ. 'yes')) THEN
	  renumber	= .TRUE.
	ELSE
	  renumber	= .FALSE.
	END IF

C Read in all information in the atoms_es.dat file.

	CALL	read_atoms_es

C Cut down the PDB file.

	CALL	pdb_to_es (molecule_coords, imc)

	CALL	EXIT
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	pdb_to_es				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	pdb_to_es (molecule_coords, imc)
C	**********	*********

C Just take the atom/hetatm records that we want, with some name changing.
C Note that we will take specific residue and atom id routines over from
C the atoms_dat collection, since we do not want to drop-out for a non-
C dictionary entry - simply list what it is and omit frim final coords.
C Still use the atoms_dat routines for the dictiobary (atoms_es.dat) read.

	LOGICAL*1	common_at
	LOGICAL		end_of_file, write_gen
	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2)
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2),
     1			buf_model (100,2)
	CHARACTER*1	buf_chain (100,2), chain_this
	CHARACTER*1	alternate, alternate_here
	CHARACTER*4	atyp, res, atyp_buf
	CHARACTER*3	res_buf
	CHARACTER*2	ion
	CHARACTER*80	atom_record
	LOGICAL		first_res, first_atom
	CHARACTER*4	res_type (200), common_atyp (100),
     1			at_type (2000), at_prtner (2000)
	REAL		at_chrg (2000,4), at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     2			at_class (2000), common_pter (100)
	LOGICAL*1	at_anyq (2000), at_ionise (2000)
	CHARACTER*3	aa (20)
	CHARACTER*80	line, file_in, file_out, lines_add (1000)
	CHARACTER*60	molecule_coords
	CHARACTER*4	atyp_here, check, atm
	CHARACTER*1	chain_here, what_atom,
     1			code_insert, code_insert_last
	LOGICAL		header_found, unfound, first_alternate,
     1			insert_seen
	LOGICAL		nucleic_here, protein_here, write_here,
     1			keep_chain, renumber
	LOGICAL		protein_out, nucleic_out, other_out,
     1			buf_write (100), ct_found, first_hyd,
     2			first_hoh, found_zero, tyr_ionise,
     3			keep_mdl
	REAL		ct_xyz (3), ot1_xyz (3), ot2_xyz (3)
	INTEGER		ct_nres
	CHARACTER*4	ct_res
	CHARACTER*1	ct_chain, chain_out, chain_zero,
     1			chain_last, code_last

	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner
	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/common_atoms/	common_atyp, common_pter, n_common
	COMMON	/files/		file_in, file_out
	COMMON	/params/	other_out, protein_out, nucleic_out,
     1				tyr_ionise, keep_chain, renumber
	COMMON	/mdl_numbers/	buf_model, keep_mdl

	DATA	aa / 'ALA','ARG','ASP','CYS','GLU','GLY','HIS',
     1		     'ILE','LEU','LYS','MET','PRO','SER','THR',
     2		     'TRP','TYR','VAL','ASN','GLN','PHE' /

1020	FORMAT (20X,'Information for pdb_es running on PDB set ',A4)
1050	FORMAT (A80)
1060	FORMAT (A1)
1100	FORMAT (' ','residue ',A4,' not found in dictionary - omitted')
1110	FORMAT (/,' WATER molecule(s) detected and REMOVED')
1140	FORMAT (/,' HYDROGEN atom(s)  detected and REMOVED')
1200	FORMAT (' ','atom ',A4,' not found in dictionary residue ',A4,
     1' - whole residue omitted')
1400	FORMAT (/,'On the FIRST PASS we have reduced PDB file to
     1 one header/coord records',/,' with ',I4,' alternate',
     2' locations resolved in favour of that first encountered',/,
     3 ' and ',I5,' hydrogen atoms removed')
1440	FORMAT (/,'On the SECOND PASS we will: ',/,
     1 ' Remove residues/atoms not in the dictionary file',/,
     2 ' Output residues of type specified (protein etc.)',/,
     3 ' Rename and order perceived NT (N of nres=1) and CT (OXT)',/)
1500	FORMAT (A4, 9X,         A4, A4, A1, I4, 4X, 3F8.3)
1520	FORMAT (A4, 2X, I5, 2X, A4, A4, A1, I4, 4X, 3F8.3)
1600	FORMAT (' NT group formatted/numbered for residue ',A1,1X,A4)
1700	FORMAT (' CT group formatted/numbered for residue ',A1,1X,A4)
1800	FORMAT (/,' Done with ',I5,' residues read from intermediate
     1 file and ',I5,' written finally')
1900	FORMAT (' with ',I5,' NT and ',I5,' CT configured')
2000	FORMAT (' and  ',I5,' HOH waters removed')
2100	FORMAT (' ** NRES=0 detected (no NT then) for ',A1,1X,I4,1X,A4)
2500	FORMAT (A1)
2600	FORMAT (I4)
2700	FORMAT (/,'*** Res Insertion ***: move to end of coords',/)
2800	FORMAT (I5)
3100	FORMAT (A1)
3200	FORMAT (I4)

	insert_seen	= .FALSE.
	nats_insert	= 0

	OPEN (UNIT=9, NAME='pdb_es.inf', TYPE='NEW')
	WRITE (9,1020) molecule_coords (1:4)

C Our current atoms_dat routines read residues from an atom/hetatm list,
C so in the first pass here we will make an intermediate file with a
C single header record and a coordinate list - then process this.
C In the first pass, we will also remove alternate locations by picking
C only the first alt loc indicator that we see - across whole molecule.
C *** CLEARLY we should be updating atom_id to look only at proper field ***

C Also exclude H atoms in the first pass.
C Also change HETATM/MSE records to ATOM/MET, with SE -> SG change.

	IF (keep_mdl) n_model	= 1
	OPEN (UNIT=2, NAME=file_in, TYPE='OLD')
	OPEN (UNIT=1, NAME='pdb_es_temp.pdb', TYPE='NEW')
	header_found		= .FALSE.
	first_alternate		= .TRUE.
	n_alternate		= 0
	first_hyd		= .TRUE.
	n_hyd			= 0
	n_nt			= 0
	n_ct			= 0
	n_hoh			= 0
	first_hoh		= .TRUE.

	chain_last		= '9'
	code_last		= '9'
	nres_last		= 9999
	nres_count		= 9999

140	READ (2,1050,END=180) line
	IF 	(line (1:6) .EQ. 'HEADER') THEN
	  IF (.NOT. header_found) THEN
	    WRITE (1,1050) line
	    header_found	= .TRUE.
	  END IF
	ELSE IF (line(1:6).EQ.'ENDMDL') THEN
	  IF (keep_mdl) THEN
	    n_model		= n_model + 1
	  END IF
	ELSE IF ((line (1:6).EQ.'ATOM  ') .OR.
     1		 (line (1:6).EQ.'HETATM')) THEN

C (Feb 2003) Put in a single header record if we havent already got one.
	  IF (.NOT. header_found) THEN
	    WRITE (1,*) 'HEADER  coordinate file'
	    header_found	= .TRUE.
	  END IF

	  IF (keep_mdl) THEN	! subvert atom index if (keep_mdl) only
	    WRITE (line(7:11),2800) n_model
	  END IF

C Nov 03 - change ang MSE to MET before further processing.
	  IF (line (18:20).EQ. 'MSE') THEN
	    line (1:6) 		= 'ATOM  '
	    line (18:20) 	= 'MET'
	    IF (line (13:15) .EQ. 'SE ') THEN
	      line (13:15)	= ' SD'
	    END IF
	  END IF

C Jun 06 (later) - actually do 2 H checks - before/after moving outliers.

	  READ (line (14:14),1060) what_atom
	  IF (what_atom .EQ. 'H') THEN
	    n_hyd		= n_hyd + 1
	    IF (first_hyd) THEN
	      WRITE (9,1140)
	      first_hyd		= .FALSE.
	    END IF
	    GOTO 140
	  END IF

C Jun 06 - move the atom name move to before H check - so we don't miss H.

	  IF (line (13:13) .NE. ' ') THEN
	    atyp_buf (1:4)	= line (13:16)
	    line (14:17)	= atyp_buf (1:4)
	    line (13:13)	= ' '
	  END IF

	  READ (line (14:14),1060) what_atom
	  IF (what_atom .EQ. 'H') THEN
	    n_hyd		= n_hyd + 1
	    IF (first_hyd) THEN
	      WRITE (9,1140)
	      first_hyd		= .FALSE.
	    END IF
	    GOTO 140
	  END IF

	  READ (line (17:17),1060) alternate_here
	  IF (alternate_here .NE. ' ') THEN
	    IF (first_alternate) THEN
	      alternate	= alternate_here
	      first_alternate = .FALSE.
	    END IF
	    IF (alternate_here .EQ. alternate) THEN
	      line (17:17) = ' '
	    ELSE
	      n_alternate	= n_alternate + 1
	      GOTO 140
	    END IF
	  END IF

	  IF (line (18:18) .EQ. ' ') THEN
	    ion			= line (19:20)
	    IF ((ion.EQ.'CA').OR.(ion.EQ.'SI').OR.
     1		(ion.EQ.'NA').OR.(ion.EQ.'CL').OR.
     2		(ion.EQ.'FE').OR.(ion.EQ.'ZN').OR.
     3		(ion.EQ.'CU').OR.(ion.EQ.'MG')) THEN
	      res_buf (1:3)	= line (19:21)
	      line (18:20)	= res_buf (1:3)
	      line (21:21)	= ' '
	    END IF
	  END IF

C Oct 2003 add in to gather up residue insertions, and fudge into a single
C disjointed chain at the end of the coordinate file.
C July 2008 - option (from SAME_CHAIN env variable) to not rename chain.
C March 2009 - option (INSERT_RENUMBER) for a bigger change, where we keep
C March 2009 - the same chain and renumber to maintain the aa sequence order.

C March 2009 - ALSO write a little program to check/enforce the register
C March 2009 - between comp_modeller.align and comp_modeller.pdb

	  READ (line(27:27),2500) code_insert
	  IF (code_insert .EQ. ' ') THEN

	    IF (renumber) THEN
	      READ (line(22:22),3100) chain_here
	      READ (line(23:26),3200) nres_here
	      IF (chain_here .EQ. chain_last) THEN
	        IF ((nres_here .NE. nres_last) .OR.
     1		    (code_insert .NE. code_last)) THEN
		  IF (nres_here .GT. (nres_last + 1)) THEN
		    nres_count	= nres_here
		  ELSE
		    nres_count	= nres_count + 1
		  END IF
		END IF
		IF (nres_count .GE. 9999) STOP '*** nres count wrong'
		WRITE (line(23:26),3200) nres_count
	      ELSE			! new chain - nres counter reset
	        nres_count	= nres_here
	      END IF
	      nres_last		= nres_here
	      code_last		= code_insert
	      chain_last	= chain_here
	    END IF
	    WRITE (1,1050) line

	  ELSE

	    IF (renumber) THEN

	      line(27:27)	= ' '		! zero insertion code
	      READ (line(22:22),3100) chain_here
	      READ (line(23:26),3200) nres_here
	      IF (chain_here .EQ. chain_last) THEN
	        IF ((nres_here .NE. nres_last) .OR.
     1		    (code_insert .NE. code_last)) THEN
		  IF (nres_here .GT. (nres_last + 1)) THEN
		    nres_count	= nres_here
		  ELSE
		    nres_count	= nres_count + 1
		  END IF
		END IF
		IF (nres_count .GE. 9999) STOP '*** nres count wrong'
		WRITE (line(23:26),3200) nres_count
	      ELSE			! new chain - nres counter reset
	        nres_count	= nres_here
	      END IF
	      nres_last		= nres_here
	      code_last		= code_insert
	      chain_last	= chain_here
	      WRITE (1,1050) line

	    ELSE

	      READ (line(23:26),2600) nres_insert
	      IF (.NOT.insert_seen) THEN
	        WRITE (6,2700)
	        WRITE (9,2700)
	        nres_insert_here	= 2002
	        insert_seen		= .TRUE.
	      ELSE
	        IF ((nres_insert.NE.nres_insert_last).OR.
     1		    (code_insert.NE.code_insert_last)) THEN
		  nres_insert_here = 2 + nres_insert_here
	        END IF
	      END IF
	      code_insert_last	= code_insert
	      nres_insert_last	= nres_insert
	      WRITE (line(23:26),2600) nres_insert_here
	      IF (.NOT. keep_chain) THEN
	        line (22:22)	= 'Z'
	      END IF
C??	      line (22:22)	= 'Z'
	      nats_insert		= nats_insert + 1
	      IF (nats_insert .GT. 1000) STOP '>1000 atom/res inserts'
	      lines_add (nats_insert) = line

	    END IF

	  END IF

	END IF
	GOTO 140

180	WRITE (9,1400) n_alternate, n_hyd
	WRITE (9,1440)
	DO 190 nins		= 1, nats_insert
	  line			= lines_add (nins)
	  WRITE (1,1050) line
190	CONTINUE						! End nins

C Now onto the second pass and coordinate renaming/shuffling.
C The atoms_dat routines expect input coordinates on unit 1.

	CLOSE (UNIT=2)
	REWIND 1
	OPEN (UNIT=3, NAME=file_out, TYPE='NEW')
	READ (1,1050) line
	WRITE (3,1050) line

	ir_read		= 0
	ir_write	= 0
	found_zero	= .FALSE.
	end_of_file	= .FALSE.
200	ir_read		= ir_read + 1
	CALL	read_residue (end_of_file)
	res		= buf_res (1,2)

C Check for tyrosine swapping from TYR to fixed/OH.

	IF (.NOT. tyr_ionise) THEN
	  DO 220 ia	= 1, n_buf (2)
	    IF (buf_res (ia,2) .EQ. 'TYR ') buf_res (ia,2) = 'TYRH'
220	  CONTINUE
	END IF

C March.03 Set the zero nres found check for avoiding NT shenanigans.
C Unset found_zero when the chain changes.

	IF (buf_nres (1,2) .EQ. 0) THEN
	  found_zero	= .TRUE.
	  WRITE (9,2100) buf_chain (1,2),buf_nres (1,2),buf_res (1,2)
	  chain_zero	= buf_chain (1,2)
	END IF
	IF (found_zero) THEN
	  IF (buf_chain (1,2) .NE. chain_zero) found_zero = .FALSE.
	END IF

C Residue/atom id routines check that we have a dictionary entry.

	CALL	res_id_here (res, irt, ia_1, ia_2, na, unfound)
	IF (unfound) THEN
	  IF (res .EQ. 'HOH ') THEN
	    IF (first_hoh) THEN
	      WRITE (9,1110)
	      first_hoh	= .FALSE.
	    END IF
	    n_hoh	= n_hoh + 1
	  ELSE
	    WRITE (9,1100) res
	  END IF
	  GOTO 680
	END IF

C	IF (n_buf (2) .GT. na) STOP 'too many atoms in residue'
C We may have one extra OXT in here, beyond eg the PRO atom count.
	IF (n_buf (2) .GT. (na+1)) THEN
	  WRITE (6,*) buf_chain (1,2),' ',
     1	              buf_nres (1,2),' ',buf_res (1,2)
	END IF
	IF (n_buf (2) .GT. (na+1)) STOP 'too many atoms in residue'

	DO 300 ia	= 1, n_buf (2)
	  atyp		= buf_atyp (ia,2)
	  common_at	= .FALSE.
	  DO 240 i_com	= 1, n_common
	    IF (atyp .EQ. common_atyp (i_com)) THEN
	      iat	= common_pter (i_com)
	      common_at	= .TRUE.
	    END IF
240	  CONTINUE						! End i_com
	  IF (.NOT. common_at) THEN
	    CALL	atom_id_here (atyp, iat, ia_1,ia_2, unfound)
	    IF (unfound) THEN
	      WRITE (9,1200) atyp, res
	      GOTO 680
	    END IF
	  END IF
300	CONTINUE						! End ia

C Now decide if we are going to write out this residue, and whether to play
C around with it (e.g. Nt, Ct) - only dictionary residues will have got here.
C Actually HOH is not currently in atoms_es.dat.
C *** Ideally we would have a type flag within the dictionary entry ***

	nucleic_here	= .FALSE.
	IF ((res.EQ.'  A ').OR.(res.EQ.'  T ').OR.
     1	    (res.EQ.'  G ').OR.(res.EQ.'  C ').OR.
     2	    (res.EQ.'  U ')) THEN
	  nucleic_here	= .TRUE.
	END IF

	protein_here	= .FALSE.
	DO 400 iaa	= 1, 20
	  IF ( res (1:3) .EQ. aa (iaa)) THEN
	    protein_here = .TRUE.
	    GOTO 410
	  END IF
400	CONTINUE						! End iaa

410	write_here	= .FALSE.
	IF	(protein_here) THEN
	  IF (protein_out) write_here = .TRUE.
	ELSE IF (nucleic_here) THEN
	  IF (nucleic_out) write_here = .TRUE.
	ELSE
	  IF (other_out) write_here = .TRUE.
	END IF

C Process now Nt, Ct (if protein) by first gathering up and flagging
C buf slots to avoid in current residue write, and then writing.
C March.03 - put in a check to see if we have already an nres=0 on current
C chain - if so just skip the NT fudge altogether - avoids dropping out.

	IF (write_here) THEN
	ir_write		= ir_write + 1

	ct_found		= .FALSE.
	DO 600 ia		= 1, n_buf (2)
	  buf_write (ia)	= .TRUE.
	  atyp			= buf_atyp (ia,2)
	  nres			= buf_nres (ia,2)
	  IF ((nres .EQ. 1) .AND. (atyp .EQ. 'N   ')) THEN
	    IF (.NOT. found_zero) THEN
	      buf_write (ia)	= .FALSE.
	      n_nt		= n_nt + 1
	      IF (buf_chain (ia,2) .EQ. ' ') THEN
	        chain_out		= '-'
	      ELSE
	        chain_out		= buf_chain (ia,2)
	      END IF
	      IF (keep_mdl) THEN
		mdl_here	=	 buf_model (ia,2)
	        WRITE (3,1520) 'ATOM',mdl_here,'NT  ',buf_res (ia,2),
     1			   chain_out,0,(buf_xyz (ia,n,2),n=1,3)
	      ELSE
	        WRITE (3,1500) 'ATOM', 'NT  ', buf_res (ia,2),
     1			   chain_out,0,(buf_xyz (ia,n,2),n=1,3)
	      END IF
	      WRITE (9,1600) chain_out, buf_res (ia,2)
	    END IF
	  END IF
	  IF (atyp .EQ. 'OXT ') THEN
	    ct_found		= .TRUE.
	    buf_write (ia)	= .FALSE.
	    n_ct_bits		= 1
	    ct_nres		= nres + 1
	    ct_res		= buf_res (ia,2)
	    ct_chain		= buf_chain (ia,2)
	    DO 520 n		= 1, 3
	      ot2_xyz (n)	= buf_xyz (ia,n,2)
520	    CONTINUE						! End n
C from CT	    IF (keep_mdl) mdl_here	= buf_model (ia,2)
	    DO 560 ja		= 1, n_buf (2)
	      IF      (buf_atyp (ja,2) .EQ. 'C   ') THEN
		buf_write (ja)	= .FALSE.
		n_ct_bits	= n_ct_bits + 1
		DO 530 n	= 1, 3
		  ct_xyz (n)	= buf_xyz (ja,n,2)
530		CONTINUE					! End n
		IF (keep_mdl) mdl_here	= buf_model (ja,2)
	      ELSE IF (buf_atyp (ja,2) .EQ. 'O   ') THEN
		buf_write (ja)	= .FALSE.
		n_ct_bits	= n_ct_bits + 1
		DO 540 n	= 1, 3
		  ot1_xyz (n)	= buf_xyz (ja,n,2)
540		CONTINUE					! End n
C from CT		IF (keep_mdl) mdl_here	= buf_model (ja,2)
	      END IF
560	    CONTINUE						! End ja
	    IF (n_ct_bits .NE. 3) STOP ' at CT without 3 atoms'
	  END IF
600	CONTINUE						! End ia

	DO 660 ia		= 1, n_buf (2)
	  IF (buf_write (ia)) THEN
	    CALL write_atom_here (ia, 2)
	  END IF
660	CONTINUE						! End ia

C Write out any CT from that last processed residue.

	IF (ct_found) THEN
	  n_ct			= n_ct + 1
	  IF (ct_chain .EQ. ' ') THEN
	    chain_out		= '-'
	  ELSE
	    chain_out		= ct_chain
	  END IF
	  IF (keep_mdl) THEN
	   WRITE(3,1520) 'ATOM',mdl_here,'CT  ',ct_res,chain_out,
     1			 ct_nres,ct_xyz
	   WRITE(3,1520) 'ATOM',mdl_here,'OT1 ',ct_res,chain_out,
     1			 ct_nres,ot1_xyz
	   WRITE(3,1520) 'ATOM',mdl_here,'OT2 ',ct_res,chain_out,
     1			 ct_nres,ot2_xyz
	  ELSE
	   WRITE(3,1500) 'ATOM','CT  ',ct_res,chain_out,ct_nres,ct_xyz
	   WRITE(3,1500) 'ATOM','OT1 ',ct_res,chain_out,ct_nres,ot1_xyz
	   WRITE(3,1500) 'ATOM','OT2 ',ct_res,chain_out,ct_nres,ot2_xyz
	  END IF
	  WRITE (9,1700) chain_out, ct_res
	  ct_found		= .FALSE.
	END IF

	END IF							! write_here

680	IF (first_res) first_res =.FALSE.
	IF (end_of_file) THEN
	  GOTO 700
	ELSE
	  GOTO 200
	END IF

700	CLOSE (UNIT=1)
	CLOSE (UNIT=3)
	WRITE (9,1800) ir_read, ir_write
	WRITE (9,1900) n_nt, n_ct
	WRITE (9,2000) n_hoh
	CLOSE (UNIT=9)
	RETURN
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	write_atom_here				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	write_atom_here (is, ib)
C	**********	***************

C Write out the atom in slot is of buffer ib.

	CHARACTER*4	buf_atyp (100,2), buf_res (100,2),
     1			buf_atom (100,2)
	REAL		buf_xyz (100,3,2)
	INTEGER		buf_nres (100,2), n_buf (2),
     1			buf_model (100,2)
	CHARACTER*1	buf_chain (100,2), chain_here, chain_out
	CHARACTER*80	atom_record
	CHARACTER*4	atyp_here, res_here
	LOGICAL		first_res, first_atom, keep_mdl

	COMMON	/coord_buf/	buf_atyp, buf_res, buf_xyz,
     1				buf_atom, buf_nres, first_res,
     2				atom_record, n_buf, first_atom,
     3				buf_chain
	COMMON	/mdl_numbers/	buf_model, keep_mdl

1020	FORMAT (A4, 9X,         A4, A4, A1, I4, 4X, 3F8.3)
1040	FORMAT (A4, 2X, I5, 2X, A4, A4, A1, I4, 4X, 3F8.3)

	nres_here	= buf_nres (is,ib)
	atyp_here	= buf_atyp (is,ib)
	res_here	= buf_res (is,ib)
	chain_here	= buf_chain (is,ib)

	IF (buf_chain(is,ib) .EQ. ' ') THEN
	  chain_out	= '-'
	ELSE
	  chain_out	= buf_chain(is,ib)
	END IF
	IF (keep_mdl) THEN
	  WRITE (3,1040) buf_atom (is,ib), buf_model (is,ib),
     1					   buf_atyp (is,ib),
     2		         buf_res (is,ib), chain_out, nres_here,
     3		        (buf_xyz (is,n,ib), n=1,3)
	ELSE
	  WRITE (3,1020) buf_atom (is,ib), buf_atyp (is,ib),
     1		         buf_res (is,ib), chain_out, nres_here,
     2		        (buf_xyz (is,n,ib), n=1,3)
	END IF
	RETURN
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	res_id_here				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	res_id_here (res, irt, ia_1,ia_2, na, unfound)
C	**********	***********

C Return the index (irt) of residue (A4 type res), and the atom index limits.
C Copied over from atoms_dat routines so that we do not drop-out on missing.

	CHARACTER*4	res_type (200), at_type (2000),
     1			at_prtner (2000), res
	REAL		at_chrg(2000,4), at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     1			at_class (2000)
	LOGICAL*1	at_anyq (2000), at_ionise (2000)
	LOGICAL		unfound

	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner

	unfound		= .FALSE.
	DO 300 i	= 1, n_res_types
	  IF (res .EQ. res_type (i)) THEN
	    irt		= i
	    ia_1	= nat_pter (i,1)
	    ia_2	= nat_pter (i,2)
	    na		= n_res_ats (i)
	    RETURN
	  END IF
300	CONTINUE						! End i
	unfound		= .TRUE.
	RETURN
	CALL	EXIT
	END


C-----------------------------------------------------------------------------C
C									      C
C		SUBROUTINE	atom_id_here				      C
C									      C
C-----------------------------------------------------------------------------C

	SUBROUTINE	atom_id_here (atyp, iat, ia_1,ia_2, unfound)
C	**********	************

C Return the atom index iat for this atom (atyp) between ia_1, ia_2.
C Copied over from atoms_dat routines so that we do not drop-out on missing.

	CHARACTER*4	res_type (200), at_type (2000),
     1			at_prtner (2000), atyp
	REAL		at_VdW (100,10), at_chrg (2000,4),
     1			at_pK (2000)
	INTEGER		n_res_ats (200), nat_pter (200,2),
     1			at_class (2000)
	CHARACTER*34	at_info (100)
	LOGICAL*1	at_anyq (2000), at_ionise (2000)
	LOGICAL		unfound

	COMMON	/atoms_es_type/ at_info, n_at_types, at_VdW, n_VdW
	COMMON	/atoms_es_resi/	n_res_types, n_chrg, n_all_at,
     1				res_type, n_res_ats, nat_pter,
     2				at_type, at_class, at_chrg,
     3				at_anyq, at_ionise, at_pK,
     4				at_prtner

	unfound		= .FALSE.
	DO 300 i	= ia_1, ia_2
	  IF (atyp .EQ. at_type (i)) THEN
	    iat		= i
	    RETURN
	  END IF
300	CONTINUE						! End i
	unfound		= .TRUE.
	RETURN
	CALL	EXIT
	END


	SUBROUTINE	leader (char_array, itop)
C	**********	******

	CHARACTER*60	char_array

	itop	= 1
	DO WHILE (char_array (itop:itop) .NE. ' ')
	  itop	= itop + 1
	END DO
	itop	= itop - 1

	RETURN
	END

